#version 430


uniform sampler2D depthMap;

in VertexData {
	vec3 position_world;
	vec3 normal_world;
	vec2 uv;
	vec4 fragPosLightSpace;
} vert;

out vec4 fragColor;


uniform vec3 camera_world;
uniform float specularAlpha;
uniform vec3 materialCoefficients; // x = ambient, y = diffuse, z = specular 


uniform sampler2D texture_diffuse0;
uniform sampler2D texture_diffuse1;
uniform sampler2D texture_specular1;
uniform sampler2D texture_normal1;
uniform sampler2D texture_height1;

uniform float brightness;


uniform struct DirectionalLight {
	vec3 color;
	vec3 direction;
} dirL;



float ShadowCalculation(vec4 fragPosLightSpace)
{
	// perform perspective divide
    vec3 projCoords = fragPosLightSpace.xyz / fragPosLightSpace.w;
    
	// transform to [0,1] range
    projCoords = projCoords * 0.5 + 0.5;
   
    // get closest depth value from light's perspective (using [0,1] range fragPosLight as coords)
    //float pcfDepth = textureProj(depthMap, fragPosLightSpace); 
    
	// get depth of current fragment from light's perspective
    float currentDepth = projCoords.z;
    
	// check whether current frag pos is in shadow
    //float shadow = currentDepth > pcfDepth  ? 1.0 : 0.0;

	float shadow = 0.0;
	vec2 texelSize = 1.0 / textureSize(depthMap, 0);
	for(int x = -1; x <= 1; ++x)
	{
		for(int y = -1; y <= 1; ++y)
		{
			float pcfDepth = texture(depthMap, projCoords.xy + vec2(x, y) * texelSize).r; 
			shadow += currentDepth > pcfDepth ? 1.0 : 0.0;        
		}    
	}
	shadow /= 9.0;

    return shadow;
}


vec3 phong(vec3 n, vec3 l, vec3 v, vec3 diffuseC, float diffuseF, vec3 specularC, float specularF, float alpha, bool attenuate, vec3 attenuation) {
	float d = length(l);
	l = normalize(l);
	float att = 1.0;	
	if(attenuate) att = 1.0f / (attenuation.x + d * attenuation.y + d * d * attenuation.z);
	vec3 r = reflect(-l, n);
	float shadow = ShadowCalculation(vert.fragPosLightSpace);       
	return (1.0 - shadow) * (diffuseF * diffuseC * max(0, dot(n, l)) + ((1.0 - shadow) * specularF * specularC * pow(max(0, dot(r, v)), alpha)) * att); 
}



void main() {


	vec3 n = normalize(vert.normal_world);
	vec3 v = normalize(camera_world - vert.position_world);

	vec3 texColor = texture(texture_diffuse0, vert.uv).rgb * brightness;
	fragColor = vec4(texColor*materialCoefficients.x, 1);

	// add directional light contribution
	fragColor.rgb += phong(n, -dirL.direction, v, dirL.color * texColor, materialCoefficients.y, dirL.color, materialCoefficients.z, specularAlpha, false, vec3(0));
	

}