#version 430


layout (location = 0) in vec3 position;
layout (location = 1) in vec3 normal;
layout (location = 2) in vec2 uv;
layout (location = 3) in ivec4 boneIDs;
layout (location = 4) in vec4 weights;

out VertexData {
	vec3 position_world;
	vec3 normal_world;
	vec2 uv;
	vec4 fragPosLightSpace;
} vert;

const int MAX_BONES = 100;

uniform mat4 modelMatrix;
uniform mat4 viewProjectionMatrix;
uniform mat3 normalMatrix;
uniform mat4 lightSpaceMatrix;

uniform mat4 bones[MAX_BONES];


void main() {
	mat4 boneTransform = bones[boneIDs[0]] * weights[0];
	boneTransform += bones[boneIDs[1]] * weights[1];
	boneTransform += bones[boneIDs[2]] * weights[2];
	boneTransform += bones[boneIDs[3]] * weights[3];
	
	vec4 normal_ = boneTransform * vec4(normal, 0);
	vert.normal_world = normalMatrix * normal_.xyz;
	vert.uv = uv;

	vec4 position_ = modelMatrix * boneTransform * vec4(position,1); // modelMatrix?
	vert.position_world = (1 * position_).xyz;

	vert.fragPosLightSpace = lightSpaceMatrix * vec4(vert.position_world, 1.0f);

	gl_Position = viewProjectionMatrix * position_;
	
}